from hm_pyhelper.miner_param import get_mac_address
import unittest
from unittest.mock import mock_open, patch
import sys
sys.path.append("..")


class TestGetEthMac(unittest.TestCase):
    def test_get_data(self):
        m = mock_open(read_data='Unknown')
        with patch('builtins.open', m):
            result = get_mac_address("random/path")
            self.assertEqual(result, 'UNKNOWN')

    def test_available_file(self):
        m = mock_open()
        m.side_effect = PermissionError
        with patch("builtins.open", mock_open()) as mf:
            fh_mock = mf.return_value.__enter__.return_value
            fh_mock.write.side_effect = FileNotFoundError
            get_mac_address("random/path")
            self.assertRaises(FileNotFoundError)

    def test_permissions_error(self):
        m = mock_open()
        m.side_effect = PermissionError
        with patch("builtins.open", mock_open()) as mf:
            fh_mock = mf.return_value.__enter__.return_value
            fh_mock.write.side_effect = PermissionError
            get_mac_address("random/path")
            self.assertRaises(PermissionError)

    def test_types(self):
        self.assertRaises(TypeError, get_mac_address, 1)
        self.assertRaises(TypeError, get_mac_address, ["123", "456"])



import os
import pytest
from unittest import TestCase
from unittest.mock import patch, mock_open

from hm_pyhelper.exceptions import UnknownVariantException, \
    UnknownVariantAttributeException

from hm_pyhelper.hardware_definitions import is_rockpi, variant_definitions, \
    get_variant_attribute, is_raspberry_pi, is_bobcat_px30
from hm_pyhelper.sbc import BALENA_ENV_RASPBERRY_PI_MODELS, \
    BALENA_ENV_ROCKPI_MODELS, BALENA_ENV_BOBCATPX30_MODELS

BUILTINS_OPEN_LITERAL = "builtins.open"


class TestHardwareDefinitions(TestCase):
    def test_variant_definitions(self):
        # Not currently expecting APPNAME, GPIO_PIN_LED, or PIO_PIN_BUTTON
        expected_fields = {
            'FRIENDLY',
            'SPIBUS',
            'RESET',
            'MAC',
            'STATUS',
            'BUTTON',
            'ECCOB',
            'TYPE',
            'CELLULAR',
            'FCC_IDS',
            'CONTAINS_FCC_IDS',
            'IC_IDS',
            'CONTAINS_IC_IDS'
        }

        for variant_name, variant_dict in variant_definitions.items():
            variant_keys = variant_dict.keys()
            missing_keys = expected_fields.difference(variant_keys)
            self.assertSetEqual(missing_keys, set())

    mock_variant_definitions = {
        'NEBHNT-XYZ': {
            'FRIENDLY': 'XYZ Hotspot Gen 1',
            'APPNAME': 'APPNAMEXYZ',
            'SPIBUS': 'spidevX.Y',
            'KEY_STORAGE_BUS': '/dev/i2c-X',
            'RESET': 00,
            'MAC': 'ethXYZ',
            'STATUS': 00,
            'BUTTON': 00,
            'ECCOB': True,
            'TYPE': 'TYPEXYZ',
            'CELLULAR': False,
            'FCC_IDS': ['1'],
            'CONTAINS_FCC_IDS': ['2', '3'],
            'IC_IDS': ['4'],
            'CONTAINS_IC_IDS': []
        }
    }

    @patch('hm_pyhelper.hardware_definitions.variant_definitions',
           mock_variant_definitions)
    def test_get_variant_attribute(self):
        mock_variant = self.mock_variant_definitions['NEBHNT-XYZ']
        mock_variant_items = mock_variant.items()
        for attribute_name, attribute_val in mock_variant_items:
            returned_val = get_variant_attribute('NEBHNT-XYZ', attribute_name)
            self.assertEqual(returned_val, attribute_val)

    @patch('hm_pyhelper.hardware_definitions.variant_definitions',
           mock_variant_definitions)
    def test_get_variant_attribute_unknown_variant(self):
        with pytest.raises(UnknownVariantException):
            get_variant_attribute('Nonexistant', 'FRIENDLY')

    @patch('hm_pyhelper.hardware_definitions.variant_definitions',
           mock_variant_definitions)
    def test_get_variant_attribute_unknown_attribute(self):
        with pytest.raises(UnknownVariantAttributeException):
            get_variant_attribute('NEBHNT-XYZ', 'Nonexistant')

    # raspberry pi model names picked from pi kernel sources
    # https://github.com/raspberrypi/linux
    # grep -ir "raspberry" linux/arch/arm*  | grep "model ="  | cut -d "=" -f2
    mock_known_dts_pi_models = [
        "Raspberry Pi Model B+",
        "Raspberry Pi Model B",
        "Raspberry Pi Compute Module",
        "Raspberry Pi Zero",
        "Raspberry Pi 2 Model B rev 1.2",
        "Raspberry Pi Compute Module 3",
        "Raspberry Pi Zero 2 W",
        "Raspberry Pi 4 Model B",
        "Raspberry Pi 400",
        "Raspberry Pi Compute Module 4",
        "Raspberry Pi Compute Module 4S",
        "Raspberry Pi Model A+",
        "Raspberry Pi Model A",
        "Raspberry Pi Model B rev2",
        "Raspberry Pi Compute Module IO board rev1",
        "Raspberry Pi Zero W",
        "Raspberry Pi 2 Model B",
        "Raspberry Pi 3 Model A+",
        "Raspberry Pi 3 Model B+",
        "Raspberry Pi 3 Model B",
        "Raspberry Pi Compute Module 3 IO board V3.0"
    ]

    def test_is_raspberry_pi(self):
        for model in self.mock_known_dts_pi_models:
            with patch(BUILTINS_OPEN_LITERAL, new_callable=mock_open, read_data=model):
                self.assertTrue(is_raspberry_pi())
            with patch(BUILTINS_OPEN_LITERAL, new_callable=mock_open, read_data="Rock something"):
                self.assertFalse(is_raspberry_pi())

        # test balena env based detection
        for model in BALENA_ENV_RASPBERRY_PI_MODELS:
            with patch.dict(os.environ, {'BALENA_DEVICE_TYPE': model}):
                self.assertTrue(is_raspberry_pi())
            # in absence of the env, it should look for /proc/device-tree/model
            # which will not exist on test environment.
            with self.assertRaises(FileNotFoundError):
                self.assertFalse(is_raspberry_pi())

    mock_known_rock_dts_models = ["ROCK PI 4B"]

    def test_is_rock_pi(self):
        for model in self.mock_known_rock_dts_models:
            with patch(BUILTINS_OPEN_LITERAL, new_callable=mock_open, read_data=model):
                self.assertTrue(is_rockpi())
            with patch(BUILTINS_OPEN_LITERAL, new_callable=mock_open,
                       read_data="raspberry something"):
                self.assertFalse(is_rockpi())

        # test balena env based detection
        for model in BALENA_ENV_ROCKPI_MODELS:
            with patch.dict(os.environ, {'BALENA_DEVICE_TYPE': model}):
                self.assertTrue(is_rockpi())
            # in absence of the env, it should look for /proc/device-tree/model
            # which will not exist on test environment.
            with self.assertRaises(FileNotFoundError):
                self.assertFalse(is_rockpi())

    mock_known_bobcat_px30_dts_models = ["Bobcat PX30"]

    def test_is_bobcat_px30(self):
        for model in self.mock_known_bobcat_px30_dts_models:
            with patch(BUILTINS_OPEN_LITERAL, new_callable=mock_open, read_data=model):
                self.assertTrue(is_bobcat_px30())
            with patch(BUILTINS_OPEN_LITERAL, new_callable=mock_open,
                       read_data="raspberry something"):
                self.assertFalse(is_bobcat_px30())

        # test balena env based detection
        for model in BALENA_ENV_BOBCATPX30_MODELS:
            with patch.dict(os.environ, {'BALENA_DEVICE_TYPE': model}):
                self.assertTrue(is_bobcat_px30())
            # in absence of the env, it should look for /proc/device-tree/model
            # which will not exist on test environment.
            with self.assertRaises(FileNotFoundError):
                self.assertFalse(is_bobcat_px30())



from typing import Dict, Union
from collections import Counter

EVENT_COUNT_KEY = "event_count"
SUPPRESSION_COUNT_KEY = "suppression_count"

# per session event counters, didn't feel the need to persist them
event_counters: Dict[str, Counter] = {}

# maximum suppression interval. It grows exponentially till this value is reached.
max_suppression_count = 128


def sentry_fingerprint(hints: Dict) -> Union[str, None]:
    """ return sha256 of log_record as fingerprint"""

    if 'log_record' not in hints:
        return None

    log_record = hints.get('log_record', None)
    fingerprint = f"{log_record.filename}:{log_record.lineno}:{log_record.funcName}"
    return fingerprint


def process_fingerprint(fingerprint: str) -> bool:
    """
    does book keeping based on event fingerprint.
    returns true if the event should be allowed to propagate
    """
    # decide whether to send or suppress
    send_event = False
    if fingerprint in event_counters:
        event_count = event_counters[fingerprint][EVENT_COUNT_KEY]
        suppression_count = event_counters[fingerprint][SUPPRESSION_COUNT_KEY]
        event_count += 1
        if event_count % suppression_count == 0:
            send_event = True
            suppression_count = min(suppression_count*2, max_suppression_count)
        event_counters[fingerprint] = Counter({
            EVENT_COUNT_KEY: event_count,
            SUPPRESSION_COUNT_KEY:  suppression_count
        })
    else:
        event_counters[fingerprint] = Counter({
            EVENT_COUNT_KEY: 1,
            SUPPRESSION_COUNT_KEY:  1
            })
        send_event = True

    return send_event


def before_send_filter(event: Dict, hints: Dict) -> Union[Dict, None]:
    """
    Event filter function for sentry_sdk_init() to be passed as its before_send argument.
    It will suppress logging events exponentially till max_suppression_count is reached.
    eg. 1, 2, 4, 8, 16, 32, 64 ....  max_suppression_count
    event instance count in this series will be passed to sentry and rest will be
    dropped.
    When an event is allowed to pass through to sentry, it will be enriched it with
    current event count for that unique event which is determined by its fingerprint
    calculated with sentry_fingerprint().
    One can find the event_count in event data @ event["extra"][event_count]

    Args:
        event: sentry event to be filtered
        hints: additional metadata from sentry for the hint. This will not be pushed
        cloud. it is only to help us with context.

    Returns:
        event: if event is allowed to propagate
        None: if event is dropped.
    """
    # process all uncaught exceptions
    if 'exception' in event:
        return event

    # if no hints, no filtering - logging and exception events will have hints.
    if not hints:
        return event

    fingerprint = sentry_fingerprint(hints)
    if not fingerprint:
        return event

    send_event = process_fingerprint(fingerprint)
    if send_event:
        event["extra"][EVENT_COUNT_KEY] = event_counters[fingerprint][EVENT_COUNT_KEY]
        return event
    else:
        return None



from logging import LogRecord
from unittest import TestCase
import hm_pyhelper.util.sentry as sentry_util
import json
import os

TESTDATA_DIR = os.path.join(os.path.dirname(__file__), "data/sentry")


class TestSentryUtil(TestCase):

    def setUp(self) -> None:
        error_logging_event_filename = os.path.join(TESTDATA_DIR, "sample_logging_event.json")
        exception_event_filename = os.path.join(TESTDATA_DIR, "sample_exception_event.json")
        logging_hints_filename = os.path.join(TESTDATA_DIR, "sample_logging_hints.json")
        self.error_logging_event = json.load(open(error_logging_event_filename, "r"))
        self.exception_event = json.load(open(exception_event_filename, "r"))
        self.logging_hints = json.load(open(logging_hints_filename, "r"))
        self.logging_hints['log_record'] = LogRecord(name='error log', level=3,
                                                     pathname='./test_file.py',
                                                     lineno=93, msg='error',
                                                     args={}, exc_info=None)

    def test_exception_event(self):
        # reset accumulated counters at start of test.
        sentry_util.event_counters = {}
        ret_value = sentry_util.before_send_filter(self.exception_event, {})
        self.assertIs(ret_value, self.exception_event)
        self.assertEqual(len(sentry_util.event_counters), 0)

    def test_error_without_hints(self):
        sentry_util.event_counters = {}
        ret_value = sentry_util.before_send_filter(self.error_logging_event, None)
        self.assertIs(ret_value, self.error_logging_event)
        self.assertEqual(len(sentry_util.event_counters), 0)

    def test_error_with_empty_hints(self):
        sentry_util.event_counters = {}
        ret_value = sentry_util.before_send_filter(self.error_logging_event, {})
        self.assertIs(ret_value, self.error_logging_event)
        self.assertEqual(len(sentry_util.event_counters), 0)

    def test_error_with_hints(self):
        sentry_util.event_counters = {}
        for count in range(1, 256):
            ret_value = sentry_util.before_send_filter(self.error_logging_event, self.logging_hints)
            if count in [1, 2, 4, 8, 16, 32, 64, 128, 256]:
                self.assertIs(ret_value['extra'][sentry_util.EVENT_COUNT_KEY], count)
            else:
                self.assertEqual(ret_value, None)



# Keys that are expected to be in diagnostics report.
BALENA_DEVICE_ID_KEY = 'BALENA_DEVICE_UUID'
BALENA_NAME_KEY = 'BALENA_DEVICE_NAME_AT_INIT'
BLUETOOTH_KEY = 'bluetooth'
DEVICE_STATUS_KEY = 'device_status'
DISK_IMAGE_KEY = 'disk_image'
ECC_KEY = 'ECC'
ERRORS_KEY = 'errors'
ETH_MAC_ADDRESS_KEY = 'eth_mac_address'
FIRMWARE_SHORT_HASH_KEY = 'firmware_short_hash'
FIRMWARE_VERSION_KEY = 'FIRMWARE_VERSION'
FREQUENCY_KEY = 'FREQ'
LORA_KEY = 'lora'
LTE_KEY = 'lte'
ONBOARDING_KEY = 'onboarding_key'
PF_KEY = 'legacy_pass_fail'
LORA_KEY = 'lora'
PUBLIC_KEY = 'public_key'
SERIAL_NUMBER_KEY = 'serial_number'
VARIANT_KEY = 'VARIANT'
WIFI_MAC_ADDRESS_KEY = 'wifi_mac_address'

# gatewayrs diagnostic keys
VALIDATOR_ADDRESS_KEY = 'validator_address'
VALIDATOR_URI_KEY = 'validator_uri'
VALIDATOR_BLOCK_HEIGHT_KEY = 'validator_height'
VALIDATOR_BLOCK_HEIGHT_SHORT_KEY = 'MH'
VALIDATOR_BLOCK_AGE = 'validator_block_age'
GATEWAY_PUBKEY_KEY = 'gateway_pubkey'
GATEWAY_REGION_KEY = 'gateway_region'
GATEWAY_REGION_SHORT_KEY = 'RE'



import unittest
import json
from hm_pyhelper.constants.diagnostics import ERRORS_KEY

from hm_pyhelper.diagnostics import Diagnostic, DiagnosticsReport
from hm_pyhelper.diagnostics.diagnostics_report import \
                                    DIAGNOSTICS_PASSED_KEY, \
                                    DIAGNOSTICS_ERRORS_KEY


class TestDiagnostic(unittest.TestCase):
    def test_record_result(self):
        diagnostic = Diagnostic('key', 'friendly_name')
        diagnostics_report = DiagnosticsReport()
        diagnostics_report.record_result('foo', diagnostic)

        self.assertDictEqual(diagnostics_report, {
            DIAGNOSTICS_PASSED_KEY: False,
            DIAGNOSTICS_ERRORS_KEY: [],
            'key': 'foo',
            'friendly_name': 'foo'
        })

    def test_record_failure(self):
        diagnostic = Diagnostic('key', 'friendly_name')
        diagnostics_report = DiagnosticsReport()
        diagnostics_report.record_failure('foo', diagnostic)
        self.assertDictEqual(diagnostics_report, {
            DIAGNOSTICS_PASSED_KEY: False,
            DIAGNOSTICS_ERRORS_KEY: ['key', 'friendly_name'],
            'key': 'foo',
            'friendly_name': 'foo'
        })

    def test_deserialization(self):
        report_str = '{"diagnostics_passed": false, "errors": ["ECC"], "ECC": false, "foo": "bar"}'  # noqa E501
        report_dict = json.loads(report_str)
        diagnostics_report = DiagnosticsReport(**report_dict)

        self.assertDictEqual(diagnostics_report, {
            DIAGNOSTICS_PASSED_KEY: False,
            DIAGNOSTICS_ERRORS_KEY: ['ECC'],
            'ECC': False,
            'foo': 'bar'
        })

    def test_get_error_messages(self):
        diagnostic1 = Diagnostic('key1', 'friendly_name1')
        diagnostic2 = Diagnostic('key2', 'friendly_name2')
        diagnostics_report = DiagnosticsReport()
        diagnostics_report.record_failure('Error1', diagnostic1)
        diagnostics_report.record_failure('Error2', diagnostic2)

        actual_msgs = diagnostics_report.get_error_messages()
        expected_msgs = "key1 Error: Error1\nfriendly_name1 Error: Error1" + \
            "\nkey2 Error: Error2\nfriendly_name2 Error: Error2"
        self.assertEqual(actual_msgs, expected_msgs)

    def test_get_report_subset(self):
        diagnostics_report = DiagnosticsReport()
        diagnostics_report['VA'] = 'NEBHNT-IN1'
        diagnostics_report['foo'] = 'bar'

        self.assertDictEqual(diagnostics_report.get_report_subset(["VA"]), {
            'VA': 'NEBHNT-IN1'
        })

    def test_passed_success(self):
        report = DiagnosticsReport()
        report[DIAGNOSTICS_PASSED_KEY] = True

        self.assertTrue(report.passed())

    def test_passed_false_on_errors(self):
        keys_to_test = {
            'ECC', 'onboarding_key', 'eth_mac_address', 'wifi_mac_address',
            'public_key', 'bluetooth', 'VARIANT', 'FREQ', 'serial_number'
        }

        response = {
            'diagnostics_passed': False,
            ERRORS_KEY: ['ECC', 'BN', 'OK', 'PK', 'PF'],
            'serial_number': '0000000021aabbcc',
            'ECC': 'gateway_mfr test finished with error',
            'E0': 'F0:4C:D5:58:E0:E1',
            'eth_mac_address': 'F0:4C:D5:58:E0:E1',
            'FR': '915',
            'FREQ': '915',
            'FW': '2021.11.22.0-1',
            'FIRMWARE_VERSION': '2021.11.22.0-1',
            'VA': 'NEBHNT-OUT1',
            'VARIANT': 'NEBHNT-OUT1',
            'BT': True,
            'bluetooth': True,
            'LTE': False,
            'LOR': False,
            'lora': False,
            'OK': 'gateway_mfr exited with a non-zero status',
            'onboarding_key': 'gateway_mfr exited with a non-zero status',
            'PK': 'gateway_mfr exited with a non-zero status',
            'public_key': 'gateway_mfr exited with a non-zero status',
            'PF': False,
            'legacy_pass_fail': False
        }

        for key in keys_to_test:
            response['errors'] = [key]
            report = DiagnosticsReport.from_json_dict(response)
            self.assertFalse(report.passed())
            assert report.has_errors([key]) == {key}

        response['errors'] = set(keys_to_test)
        report = DiagnosticsReport.from_json_dict(response)
        self.assertFalse(report.passed())
        assert report.has_errors(keys_to_test) == keys_to_test

    def test_assert_diagnostics_present(self):
        diagnostics_report = DiagnosticsReport.from_json_dict({
            'foo': 'bar'
        })

        missing_keys = diagnostics_report.get_missing_keys({'foo'})

        self.assertEqual(missing_keys, set())

    def test_assert_diagnostics_not_present(self):
        diagnostics_report = DiagnosticsReport.from_json_dict({
            'foo': 'bar'
        })

        missing_keys = diagnostics_report.get_missing_keys({'missing_key'})
        self.assertEqual(missing_keys, {'missing_key'})



import unittest

from unittest.mock import patch
from hm_pyhelper.sbc import is_commercial_fleet, is_nebra_fleet


class TestSBC(unittest.TestCase):

    @patch.dict('os.environ', {"BALENA_API_URL": "https://api.cloud.nebra.com"})
    @patch.dict('os.environ', {"BALENA_APP_ID": "55"})
    def test_is_nebra_fleet_true(self):
        self.assertTrue(is_nebra_fleet())

    @patch.dict('os.environ', {"BALENA_API_URL": "https://test.com"})
    @patch.dict('os.environ', {"BALENA_APP_ID": "5500"})
    def test_is_nebra_fleet_false(self):
        self.assertFalse(is_nebra_fleet())

    @patch.dict('os.environ', {"BALENA_APP_NAME": "test-c"})
    @patch.dict('os.environ', {"BALENA_APP_ID": "87"})
    def test_is_commercial_fleet_true(self):
        self.assertTrue(is_commercial_fleet())

    @patch.dict('os.environ', {"BALENA_APP_NAME": "test"})
    @patch.dict('os.environ', {"BALENA_APP_ID": "8700"})
    def test_is_commercial_fleet_false(self):
        self.assertFalse(is_commercial_fleet())
